<?php

namespace Drupal\eca_html_parser\Plugin\Action;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Wa72\HtmlPageDom\HtmlPageCrawler;

/**
 * Get current route name.
 *
 * @Action(
 *   id = "eca_html_parser_simple",
 *   label = @Translation("HTML parser simple"),
 *   description = @Translation("Parse simple html data."),
 * )
 */
class HtmlParserSimple extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'html' => '',
      'css_selector' => '',
      'token_name' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['html'] = [
      '#type' => 'textarea',
      '#required' => TRUE,
      '#title' => $this->t('HTML content'),
      '#description' => $this->t('The HTML content to parse. This field support token replacement.'),
      '#default_value' => $this->configuration['html'],
      '#weight' => 1,
    ];

    $form['css_selector'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('CSS Selector'),
      '#description' => $this->t('CSS selector used to parse data. Example #main, #main .content, #main .content .field-name-body. This field support token replacement.'),
      '#default_value' => $this->configuration['css_selector'],
      '#eca_token_reference' => TRUE,
      '#weight' => 2,
    ];

    $form['token_name'] = [
      '#required' => TRUE,
      '#type' => 'textfield',
      '#title' => $this->t('Name of token'),
      '#default_value' => $this->configuration['token_name'],
      '#description' => $this->t('The route name value will be loaded into this specified token.'),
      '#eca_token_reference' => TRUE,
      '#weight' => 20,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['html'] = $form_state->getValue('html');
    $this->configuration['css_selector'] = $form_state->getValue('css_selector');
    $this->configuration['token_name'] = $form_state->getValue('token_name');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * Get html data.
   *
   * @return string
   *   The html data.
   */
  protected function getHtml(): string {
    $html = $this->tokenService->replaceClear($this->configuration['html']);
    if (str_starts_with($html, 'http://') || str_starts_with($html, 'https://')) {
      try {
        $html = file_get_contents($html);
      }
      catch (\Exception $e) {
        $html = '';
      }
    }
    return $html;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $html = $this->getHtml();

    if (!(strlen($html))) {
      return;
    }

    $css_selector = $this->tokenService->replaceClear($this->configuration['css_selector']);
    $c = new HtmlPageCrawler($html);
    $data = $c->filter($css_selector)->first()->html();
    if (!empty($this->configuration['token_name'])) {
      $this->tokenService->addTokenData($this->configuration['token_name'], $data);
    }
  }

}
