<?php

namespace Drupal\eca_html_parser\Plugin\Action;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Service\YamlParser;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Exception\ParseException;
use Wa72\HtmlPageDom\HtmlPageCrawler;

/**
 * Get current route name.
 *
 * @Action(
 *   id = "eca_html_parser_to_list",
 *   label = @Translation("HTML parser to list"),
 *   description = @Translation("Parse html data to array list."),
 * )
 */
class HtmlParserToList extends HtmlParserSimple {

  /**
   * The YAML parser.
   */
  protected YamlParser $yamlParser;

  /**
   * Set the YAML parser.
   *
   * @param \Drupal\eca\Service\YamlParser $yaml_parser
   *   The YAML parser.
   */
  public function setYamlParser(YamlParser $yaml_parser): void {
    $this->yamlParser = $yaml_parser;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setYamlParser($container->get('eca.service.yaml_parser'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'sub_css_selector' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['sub_css_selector'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Sub Css Selector'),
      '#description' => $this->t("Sub Css Selector in the YAML format. Example: <br> title: 'h2 > a'<br> url: 'h2 > a|attr=href'<br>date: '.field-name-drupalorg-sa-date .field-item'"),
      '#cols' => 60,
      '#rows' => 5,
      '#default_value' => $this->configuration['sub_css_selector'],
      '#weight' => 3,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['sub_css_selector'] = $form_state->getValue('sub_css_selector');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    $html = $this->getHtml();
    if (!(is_string($html) && strlen($html))) {
      return;
    }

    $css_selector = $this->tokenService->replaceClear($this->configuration['css_selector']);

    $sub_css_selector = $this->tokenService->replaceClear($this->configuration['sub_css_selector']);
    if (strlen($sub_css_selector)) {
      try {
        $sub_css_selector = $this->yamlParser->parse($sub_css_selector);
      }
      catch (ParseException $e) {
        $this->logger->error('HtmlParserToList tried parsing args parameters as YAML format, but parsing failed.');
        return;
      }
    }

    $c = new HtmlPageCrawler($html);
    $data = $c->filter($css_selector)
      ->each(function ($node) use ($sub_css_selector) {
        if (is_array($sub_css_selector) && count($sub_css_selector)) {
          $item = [];
          foreach ($sub_css_selector as $key => $selector) {
            if (str_contains($selector, '|')) {
              [$selector, $attr] = explode('|', $selector);
            }
            $filter_item = $node->filter($selector);
            if ($filter_item->count() == 0) {
              $item[$key] = '';
              continue;
            }
            if (isset($attr)) {
              if ($attr === 'text') {
                $item[$key] = $filter_item->text();
              }
              elseif (str_contains($attr, 'attr=')) {
                $attr = explode('=', $attr)[1];
                $item[$key] = $filter_item->attr($attr);
              }
              else {
                $item[$key] = $filter_item->html();
              }
            }
            else {
              $item[$key] = $filter_item->html();
            }
          }
          return $item;
        }
        else {
          return $node->html();
        }
      });

    if (!empty($this->configuration['token_name'])) {
      $this->tokenService->addTokenData($this->configuration['token_name'], $data);
    }
  }

}
